/*
 * FakeMPX13_72HFMS.asm
 *
 *  Created: 08.04.2013 12:25:39
 *  Author: yurip 
 *	based on  A P P L I C A T I O N   N O T E   A V R 3 0 5 
 *  Can be used to compile AVR Studio http://www.atmel.com
 *  version for HFM-4 can also be used for similar modules with PPM input
 *  
 */
.include "tn13def.inc"

;***** Pin definitions

.equ	RxD	=0			;Receive pin is PB0
.equ	TxD	=1			;Transmit pin is PB1

;***** Global register variables

.def	bitcnt	=R16			;bit counter
.def	temp	=R17			;temporary storage register 1
.def	temp1	=R18			;temporary storage register 2
.def	temp2	=R19			;temporary storage register 3

.def	Txbyte	=R20			;Data to be transmitted
.def	Rxbyte	=R21			;Received data

.cseg
.org 0
		rjmp RESET ; Reset Handler

;************************************************************************
; Generated by delay loop calculator
; at http://www.bretmulvey.com/avrdelay.html
;
; Delay 120 000 cycles
; 100ms at 1.2 MHz
;************************************************************************
delay_100ms:			; Should be 130 ms, but left as is 
		ldi		temp, 156
    	ldi		temp1, 215
L1: 	dec  	temp1
    	brne 	L1
    	dec  	temp
    	brne 	L1
    
		ret

;***************************************************************************
;*
;* "putchar"
;*
;* This subroutine transmits the byte stored in the "Txbyte" register
;* The number of stop bits used is set with the sb constant
;*
;* Number of words	:14 including return
;* Number of cycles	:Depens on bit rate
;* Low registers used	:None
;* High registers used	:2 (bitcnt,Txbyte)
;* Pointers used	:None
;*
;***************************************************************************
.equ		sb	=1		;Number of stop bits (1, 2, ...)

putchar:	ldi	bitcnt,9+sb	;1+8+sb (sb is # of stop bits)
			com	Txbyte		;Inverte everything
			sec			;Start bit

putchar0:	brcc	putchar1	;If carry set
			cbi	PORTB,TxD	;    send a '0'
			rjmp	putchar2	;else	

putchar1:	sbi	PORTB,TxD	;    send a '1'
			nop

putchar2:	rcall UART_delay	;One bit delay
			rcall UART_delay

			lsr	Txbyte		;Get next bit
			dec	bitcnt		;If not all bit sent
			brne	putchar0	;   send next
					;else
			ret			;   return



;***************************************************************************
;*
;* "getchar"
;*
;* This subroutine receives one byte and returns it in the "Rxbyte" register
;*
;* Number of words	:14 including return
;* Number of cycles	:Depens on when data arrives
;* Low registers used	:None
;* High registers used	:2 (bitcnt,Rxbyte)
;* Pointers used	:None
;*
;***************************************************************************

getchar:	ldi 	bitcnt,9	;8 data bit + 1 stop bit

getchar1:	sbic 	PINB,RxD	;Wait for start bit
			rjmp 	getchar1

			rcall UART_delay	;0.5 bit delay

getchar2:	rcall UART_delay	;1 bit delay
			rcall UART_delay		

			clc			;clear carry
			sbic 	PINB,RxD	;if RX pin high
			sec			;

			dec 	bitcnt		;If bit is stop bit
			breq 	getchar3	;   return
					;else
			ror 	Rxbyte		;   shift bit into Rxbyte
			rjmp 	getchar2	;   go get next

getchar3:	ret



;***************************************************************************
;*
;* "UART_delay"
;*
;* This delay subroutine generates the required delay between the bits when
;* transmitting and receiving bytes. The total execution time is set by the
;* constant "b":
;*
;*	3b + 7 cycles (including rcall and ret)
;*
;* Number of words	:4 including return
;* Low registers used	:None
;* High registers used	:1 (temp)
;* Pointers used	:None
;*
;***************************************************************************
; Some b values: 	(See also table in Appnote documentation)
;
; 1 MHz crystal:
;   9600 bps - b=14
;  19200 bps - b=5
;  28800 bps - b=2
;
; 2 MHz crystal:
;  19200 bps - b=14
;  28800 bps - b=8
;  57600 bps - b=2

; 4 MHz crystal:
;  19200 bps - b=31
;  28800 bps - b=19
;  57600 bps - b=8
; 115200 bps - b=2
;
;	b=((Fclck/BaudeRate)-23)/6

.equ	b	=80	;19200 bps @ 9.6 MHz crystal
 

UART_delay:		ldi		temp,b
UART_delay1:	dec		temp
				brne	UART_delay1

				ret

;***** Program Execution Starts Here

RESET:
.equ	StartPPM				= 'a'
.equ	PreviousFrequency		= '-'
.equ	NextFrequency			= '+'
.equ	RequestPresetFrequency	= '?'
.equ	RequestModuleName		= 'v'

;reset:	
		sbi	PORTB,TxD	;Init port pins
		sbi	DDRB,TxD

// My Program
charinput:
		rcall	getchar
;		mov 	Txbyte, Rxbyte
;		rcall putchar

caseRequestModuleName:
		cpi		Rxbyte, RequestModuleName
		brne 	caseRequestPresetFrequency
        ldi 	ZH,High(str1*2) 
        ldi 	ZL,Low(str1*2)  
ReadArray1:
        lpm		Txbyte,Z+    	
        mov 	temp2,Txbyte    
		rcall	putchar			
		cpi		temp2,0x0a		
		brne	ReadArray1
		rjmp	charinput		

caseRequestPresetFrequency:
		cpi		Rxbyte, RequestPresetFrequency
		brne 	casePreviousFrequency
        ldi 	ZH,High(str2a*2) 
        ldi 	ZL,Low(str2a*2)  
ReadArray2a:
        lpm		Txbyte,Z+    	
        mov 	temp2,Txbyte    
		rcall	putchar			
		cpi		temp2,0x01		
		brne	ReadArray2a
		rcall 	delay_100ms		
        ldi 	ZH,High(str3*2) 
        ldi 	ZL,Low(str3*2)  
ReadArray3a:
        lpm		Txbyte,Z+    	
        mov 	temp2,Txbyte    
		rcall	putchar			
		cpi		temp2,0x0a		
		brne	ReadArray3a
		rjmp	charinput		

casePreviousFrequency:
		cpi		Rxbyte, PreviousFrequency
		brne 	caseNextFrequency
        ldi 	ZH,High(str2b*2) 
        ldi 	ZL,Low(str2b*2)  
ReadArray2b:
        lpm		Txbyte,Z+    	
        mov 	temp2,Txbyte    
		rcall	putchar			
		cpi		temp2,0x08		
		brne	ReadArray2b
		rcall 	delay_100ms		
        ldi 	ZH,High(str3*2) 
        ldi 	ZL,Low(str3*2)  
ReadArray3b:
       lpm		Txbyte,Z+    	
        mov 	temp2,Txbyte    
		rcall	putchar			
		cpi		temp2,0x0a		
		brne	ReadArray3b
		rjmp	charinput		

caseNextFrequency:
		cpi		Rxbyte, NextFrequency
		brne 	caseStartPPM
        ldi 	ZH,High(str2b*2) 
        ldi 	ZL,Low(str2b*2)  
ReadArray2c:
        lpm		Txbyte,Z+    	
        mov 	temp2,Txbyte    
		rcall	putchar			
		cpi		temp2,0x08		
		brne	ReadArray2c
		rcall	delay_100ms		
        ldi 	ZH,High(str3*2) 
        ldi 	ZL,Low(str3*2)  
ReadArray3c:
        lpm		Txbyte,Z+    	
        mov 	temp2,Txbyte    
		rcall	putchar			
		cpi		temp2,0x0a		
		brne	ReadArray3c
		rjmp	charinput		

caseStartPPM:
		cpi		Rxbyte, StartPPM
		breq	sleepMC 	
		rjmp	charinput
		; sleep 
sleepMC:
		ldi		temp,0x3F
		ldi		temp1,0x00
		out		PORTB,temp
		out		DDRB,temp1
		nop
		ldi		temp,0x18
		cli
		out 	MCUCR,temp
		rjmp	sleepMC
		
str1:	.db	'7','2','F','A','K','E','H','F','M','S','1','.','0',' ', 0x0d, 0x0a
str2a:	.db	'7','2',' ',' ','A','L','L','7','2','0','0','0', 0x01, 0x00
str2b:	.db	'7','2',' ',' ','A','L','L','7','2','0','0','0', 0x08,0x00
str3:	.db	'@',0x27,0x2d, 0x0d, 0x0a,0x00


